import requests
import json

TREND_URL = "https://trends.google.com/trends/api/explore"
TREND_WIDGET_URL = "https://trends.google.com/trends/api/widgetdata/relatedsearches"
TREND_MULTILINE_URL = "https://trends.google.com/trends/api/widgetdata/multiline"


class GoogleTrendSubject:
    def __init__(
        self,
        keyword: str,
        time_filter: str = "12-m",
        nation_code: str = "ko",
        time_zone: int = -540,
        geo: str = "KR",
    ):
        self.keyword = keyword
        self.time_filter = time_filter
        self.national_code = nation_code
        self.time_zone_min_delta = time_zone
        self.geo = geo
        self.cookies = None
        self.related_query = None
        self.related_topic = None
        self.timeseries = None
        self.base = None

    def get_cookie(self):
        google_trends_url = "https://trends.google.com/trends/?geo={}".format(self.geo)
        response = requests.get(google_trends_url)
        if response.status_code == 200:
            return True, response.cookies.get_dict()
        return False, None

    def get_relate_topic(self):
        widget_list = self.base["widgets"]
        token = None
        search_request = None
        headers = {
            "User-Agent": "PostmanRuntime/7.28.0",
            "Accept": "*/*",
            "Accept-Encoding": "gzip, deflate, br",
            "Connection": "keep-alive",
            "accept-language": "ko-KR,ko;q=0.9,en-US;q=0.8,en;q=0.7",
        }

        for widget in widget_list:
            if widget["id"] is not None and widget["id"] == "RELATED_TOPICS":
                search_request = widget["request"]
                token = widget["token"]
                break
        query_param = {
            "hl": self.national_code,
            "tz": self.time_zone_min_delta,
            "req": json.dumps(search_request, separators=(",", ":")),
            "token": token,
        }
        response = requests.get(
            TREND_WIDGET_URL, params=query_param, headers=headers, cookies=self.cookies
        )
        if response.status_code == 200:
            self.related_topic = (
                json.loads(self.remove_user_less_closer(response.text))
            )["default"]
            return True, self.related_topic
        else:
            return False, None

    def get_relate_search(self):
        widget_list = self.base["widgets"]
        token = None
        search_request = None
        headers = {
            "User-Agent": "PostmanRuntime/7.28.0",
            "Accept": "*/*",
            "Accept-Encoding": "gzip, deflate, br",
            "Connection": "keep-alive",
            "accept-language": "ko-KR,ko;q=0.9,en-US;q=0.8,en;q=0.7",
        }

        for widget in widget_list:
            if widget["id"] is not None and widget["id"] == "RELATED_QUERIES":
                search_request = widget["request"]
                token = widget["token"]
                break
        query_param = {
            "hl": self.national_code,
            "tz": self.time_zone_min_delta,
            "req": json.dumps(search_request, separators=(",", ":")),
            "token": token,
        }
        response = requests.get(
            TREND_WIDGET_URL, params=query_param, headers=headers, cookies=self.cookies
        )
        if response.status_code == 200:
            self.related_query = (
                json.loads(self.remove_user_less_closer(response.text))
            )["default"]
            return True, self.related_query
        else:
            return False, None
        return True

    def get_timeseries(self):
        widget_list = self.base["widgets"]
        token = None
        search_request = None
        headers = {
            "User-Agent": "PostmanRuntime/7.28.0",
            "Accept": "*/*",
            "Accept-Encoding": "gzip, deflate, br",
            "Connection": "keep-alive",
            "accept-language": "ko-KR,ko;q=0.9,en-US;q=0.8,en;q=0.7",
        }

        for widget in widget_list:
            if widget["id"] is not None and widget["id"] == "TIMESERIES":
                search_request = widget["request"]
                token = widget["token"]
                break
        query_param = {
            "hl": self.national_code,
            "tz": self.time_zone_min_delta,
            "req": json.dumps(search_request, separators=(",", ":")),
            "token": token,
        }
        response = requests.get(
            TREND_MULTILINE_URL,
            params=query_param,
            headers=headers,
            cookies=self.cookies,
        )
        if response.status_code == 200:
            self.timeseries = (json.loads(self.remove_user_less_closer(response.text)))[
                "default"
            ]
            return True, self.timeseries
        else:
            return False, None
        return True

    @staticmethod
    def remove_user_less_closer(body):
        if body[0:5] == ")]}',":
            return body[6:]
        elif body[0:4] == ")]}'":
            return body[5:]

    def search(self, run_all=False):
        try:
            c_result, cookies = self.get_cookie()
            if c_result:
                self.cookies = cookies

            headers = {
                "User-Agent": "PostmanRuntime/7.28.0",
                "Accept": "*/*",
                "Accept-Encoding": "gzip, deflate, br",
                "Connection": "keep-alive",
                "accept-language": "ko-KR,ko;q=0.9,en-US;q=0.8,en;q=0.7",
            }
            if c_result is False:
                headers["Cookies", self.tempCookies]

            time_filter_format = "today {}".format(self.time_filter)

            if self.time_filter[-1] == "H":
                time_filter_format = "now {}".format(self.time_filter)

            req_json = {
                "comparisonItem": [
                    {
                        "keyword": self.keyword,
                        "geo": self.geo,
                        "time": time_filter_format,
                    }
                ],
                "category": 0,
                "property": "youtube",
            }
            query_param = {
                "hl": self.national_code,
                "tz": self.time_zone_min_delta,
                "req": json.dumps(req_json, separators=(",", ":")),
            }
            response = requests.get(
                TREND_URL, params=query_param, headers=headers, cookies=cookies
            )

            if response.status_code == 200:
                html = response.text
                self.base = json.loads(self.remove_user_less_closer(html))
                if run_all:
                    self.get_timeseries()
                    self.get_relate_search()
                    self.get_relate_topic()
                return True, "find end"
            else:
                print(response.text)
                return False, "error response {status}".format(
                    status=response.status_code
                )
        except Exception as e:
            print(e)
            return False, "search video exception"
