import time
import json
import requests
import datetime

from ..config.secrets import get_secrets, set_env
from ..naver_adsearch.signature_helper import Signature



NAVER_TREND_URL = "https://api.naver.com"
NAVER_KEYWORD_PATH = "/keywordstool"


class NaverADSearchSubject:
    def __init__(
        self,
        keyword: str,
        month=None,
        filename: str = None,
        naver_api_key: str = "",
        naver_secret_key: str = "",
        naver_customer_id: int = None,
    ):
        self.current = None
        self.keyword = keyword
        self.filename = filename
        self.naver_api_key = naver_api_key
        self.naver_secret_key = naver_secret_key
        self.naver_customer_id = naver_customer_id
        if month is None:
            today = datetime.date.today()
            self.month = today.month
        else:
            self.month = month

    def set_secrets(self):
        secrets = get_secrets(self.filename)
        if not secrets:
            secrets = set_env(
                self.naver_api_key, self.naver_secret_key, self.naver_customer_id
            )
        try:
            self.naver_api_key = secrets.get("NAVER_API_KEY")
            self.naver_secret_key = secrets.get("NAVER_SECRET_KEY")
            self.naver_customer_id = secrets.get("NAVER_CUSTOMER_ID")
        except Exception:
            raise Exception("Fail: Not found secrets")

    def get_header(self, method, uri):
        self.set_secrets()
        timestamp = str(round(time.time() * 1000))
        signature = Signature.generate(timestamp, method, uri, self.naver_secret_key)
        return {
            "Content-Type": "application/json; charset=UTF-8",
            "X-Timestamp": timestamp,
            "X-API-KEY": self.naver_api_key,
            "X-API-SECRET": self.naver_secret_key,
            "X-Customer": str(self.naver_customer_id),
            "X-Signature": signature,
        }

    def search(self):
        try:
            headers = self.get_header("GET", NAVER_KEYWORD_PATH)
            query_data = {
                "format": "json",
                "hintKeywords": self.keyword,
                "showDetail": 1,
                "includeHintKeywords": 1,
            }
            response = requests.get(
                NAVER_TREND_URL + NAVER_KEYWORD_PATH, params=query_data, headers=headers
            )
            if response.status_code == 200:
                self.current = json.loads(response.text)
                return True, "find"
            else:
                print("Error Code:", response.status_code, response.text)
                return False, "error"
        except Exception as e:  # noqa: E722
            print(e)
            return False, "exception"
