import re
import json
from typing import Optional, Callable
from datetime import datetime, date, timedelta

import requests
import pandas as pd
import numpy as np
from dateutil.relativedelta import relativedelta

from ..naver_adsearch import ajax as adAjax

NAVER_TREND_HOOK_URL = "https://datalab.naver.com/qcHash.naver"
NAVER_TREND_GRAPH_URL = "https://datalab.naver.com/keyword/trendResult.naver?hashKey="
NAVER_SEARCH_RELATE_KEYWORD = "https://ac.search.naver.com/nx/ac"


def contains_in_list(keywords: list, func: Callable) -> bool:
    for x in keywords:
        if func(x):
            return True
    return False


class NaverTrendHookSubject:
    def __init__(
        self,
        keyword: str,
        start_time: str,
        end_time: str,
        time_unit: str = "date",
        gender: str = "",
        age: str = "",
        **naver_secret,
    ):
        self.trend_stat = None
        self.keyword = keyword
        self.time_unit = time_unit
        self.start_time = datetime.strptime(start_time, "%Y-%m-%d")
        self.end_time = datetime.strptime(end_time, "%Y-%m-%d")
        self.is_available_date = None
        self.yesterday = date.today() - timedelta(days=1)
        self.is_last_day = self.check_end_time()
        self.gender = gender
        self.age = age
        self.naver_secret = naver_secret

    @property
    def start_time(self):
        return self._start_time

    @start_time.setter
    def start_time(self, value):
        if value <= datetime.strptime("2015-12-31 00:00:00", "%Y-%m-%d %H:%M:%S"):
            raise ValueError("Start time is available after 2016-01-01")
        self._start_time = value

    @property
    def end_time(self):
        return self._end_time

    @end_time.setter
    def end_time(self, value):
        if value.date() >= date.today():
            raise ValueError("End time is available before today")
        self._end_time = value

    @property
    def is_available_date(self):
        return self._is_available_date

    @is_available_date.setter
    def is_available_date(self, value):
        if self.end_time <= self.start_time:
            raise ValueError("End date must be greater than start date")
        self._is_available_date = True

    @property
    def time_unit(self):
        return self._time_unit

    @time_unit.setter
    def time_unit(self, value):
        if value not in ["date", "week", "month"]:
            raise ValueError("Period type is one of 'date', 'week' and 'month'")
        self._time_unit = value

    @property
    def gender(self):
        return self._gender

    @gender.setter
    def gender(self, value):
        if value not in ["f", "m", ""]:
            raise ValueError("Gender type is either 'm' or 'f'")
        self._gender = value

    @property
    def age(self):
        return self._age

    @age.setter
    def age(self, value):
        if value == "":
            self._age = value
        else:
            pattern = re.compile(r"\d+(,\d+)*")
            if pattern.match(value) is None:
                raise ValueError("Number separated by comma string")
            result = set([int(x.strip()) for x in value.split(",")])
            available_value = set(range(1, 12))
            not_available_value = result - available_value
            if not_available_value:
                raise ValueError("Available 1 to 11")
            string_result = ",".join(str(x) for x in result)
            self._age = string_result

    @property
    def naver_secret(self):
        return self._naver_secret

    @naver_secret.setter
    def naver_secret(self, value):
        self.filename = value.get("filename")
        self.naver_api_key = value.get("naver_api_key", "")
        self.naver_secret_key = value.get("naver_secret_key", "")
        self.naver_custom_id = value.get("naver_custom_id")
        self._naver_secret = value

    def check_end_time(self) -> bool:
        if self.end_time.date() == self.yesterday:
            return True
        return False

    def get_count_data(self, graph_data: list, month_total_count: int) -> list[dict]:
        last_index = len(graph_data) - 1
        if len(graph_data) < 30:
            ratio_list = np.array([e.get("value") for e in graph_data[0:last_index]])
        else:
            last_data = graph_data[last_index]
            end_time = datetime.strptime(last_data.get("period"), "%Y%m%d")
            start_time = end_time - relativedelta(months=1)
            str_start_time = start_time.strftime("%Y%m%d")
            index = 0
            for data in graph_data:
                if data.get("period") == str_start_time:
                    break
                index += 1
            ratio_list = np.array(
                [e.get("value") for e in graph_data[index:last_index]]
            )
        anker = month_total_count / sum(ratio_list)
        return [
            {
                "period": self.convert_period(e.get("period")),
                "count": round(int(e.get("value") * anker) + 5, -1),
            }
            for e in graph_data
        ]

    @staticmethod
    def change_date_to_string(
        start: datetime, end: datetime, time_unit: str
    ) -> tuple[str, str]:
        start_date, end_date = ("", "")
        if time_unit == "date":
            start_date = start.strftime("%Y%m%d")
            end_date = end.strftime("%Y%m%d")
        elif time_unit in ["week", "month"]:
            start_date = start.strftime("%Y%m")
            end_date = end.strftime("%Y%m")
        return start_date, end_date

    def get_query_hook_by_device(
        self,
        start: datetime,
        end: datetime,
        time_unit: str,
        device: str = None,
        gender: str = None,
        age: str = None,
    ) -> dict:
        start_date, end_date = self.change_date_to_string(start, end, time_unit)
        body = {
            "startDate": start_date,
            "endDate": end_date,
            "timeUnit": time_unit,
            "device": device if device is not None else "",
            "age": age if age is not None else "",
            "gender": gender if gender is not None else "",
            "queryGroups": self.keyword,
        }
        return body

    @staticmethod
    def get_headers() -> dict:
        return {
            "Content-Type": "application/x-www-form-urlencoded",
            "Referer": "https://datalab.naver.com/keyword/trendSearch.naver",
            "Origin": "https://datalab.naver.com",
            "User-Agent": "PostmanRuntime/7.28.0",
        }

    @staticmethod
    def get_stats_headers() -> dict:
        return {
            "Referer": "https://datalab.naver.com/keyword/trendSearch.naver",
            "Origin": "https://datalab.naver.com",
            "User-Agent": "PostmanRuntime/7.28.0",
        }

    @staticmethod
    def get_relate_search_keyword(keyword: str) -> list:
        """키워드 툴에서 이 리스트 전체 다시 넣어 검색해 수치를 가져옴"""
        try:
            q_params = {
                "q": keyword,
                "con": 0,
                "r_format": "json",
                "r_enc": "UTF-8",
                "r_unicode": 0,
                "t_koreng": 1,
                "q_enc": "UTF-8",
                "st": 1,
            }
            response = requests.get(NAVER_SEARCH_RELATE_KEYWORD, params=q_params)

            if response.status_code == 200:
                if response.text and len(response.text):
                    results = json.loads(response.text)
                    items = results.get("items")
                    if items:
                        keyword_list = list(
                            map(lambda i: i[0].replace(" ", "").upper(), items[0])
                        )
                        return keyword_list
                else:
                    return [keyword]
            else:
                return [keyword]
        except Exception as e:  # noqa: F841
            return [keyword]

    def get_keyword_stat(self, keywords: list):
        """
        :rtype: tuple
        bool: 실행 결과
        list: 키워드가 포함된 검색어만 필터링한 리스트
        int: 최근 한 달간 PC 및 모바일에서 키워드를 검색한 횟수
        int: 최근 한 달간 PC에서 키워드를 검색한 횟수
        int: 최근 한 달간 모바일에서 키워드를 검색한 횟수
        """
        query_keyword = ",".join(keywords[0:5])
        current_stat = adAjax.NaverADSearchSubject(
            query_keyword,
            filename=self.filename,
            naver_api_key=self.naver_api_key,
            naver_secret_key=self.naver_secret_key,
            naver_customer_id=self.naver_custom_id,
        )
        result, _ = current_stat.search()
        stat_list = current_stat.current.get("keywordList", {})
        recently_pc_month_count = 0
        recently_mobile_month_count = 0
        recently_month_count = 0

        if result and len(stat_list) > 0:
            keyword_stat = list(
                filter(lambda x: x.get("relKeyword") == self.keyword, stat_list)
            )
            if keyword_stat and len(keyword_stat) > 0:
                recently_pc_month_count = keyword_stat[0].get("monthlyPcQcCnt")
                recently_mobile_month_count = keyword_stat[0].get("monthlyMobileQcCnt")
                recently_month_count = (
                    recently_pc_month_count + recently_mobile_month_count
                )
            filtered_list = []
            for item in stat_list:
                if contains_in_list(keywords, lambda x: x == item.get("relKeyword")):
                    filtered_list.append(item)
            if len(keywords) > 5:
                next_result, second_stats_list, _, _, _ = self.get_keyword_stat(
                    keywords[5:]
                )
                if next_result:
                    filtered_list += second_stats_list
            return (
                True,
                filtered_list,
                recently_month_count,
                recently_pc_month_count,
                recently_mobile_month_count,
            )
        else:
            return False, None, None, None, None

    def search(self) -> tuple[bool, str]:
        try:
            keywords = self.get_relate_search_keyword(self.keyword)
            if contains_in_list(keywords, lambda x: x == self.keyword) is False:
                keywords = [self.keyword] + keywords
            (
                keyword_stat_result,
                keyword_stat,
                all_count,
                pc_count,
                mobile_count,
            ) = self.get_keyword_stat(keywords)
            if keyword_stat_result is True:
                all_result, all_stat = self.search_trend_hook_stat(all_count)
                pc_result, pc_stat = self.search_trend_hook_stat(pc_count, "pc")
                mobile_result, mobile_stat = self.search_trend_hook_stat(
                    mobile_count, "mo"
                )
                self.trend_stat = {
                    "all": all_stat if all_result is True else None,
                    "pc": pc_stat if pc_result is True else None,
                    "mobile": mobile_stat if mobile_result is True else None,
                    "relKeywords": keyword_stat,
                }
                return True, "find"
            else:
                return False, "error KeywordStat Getting"
        except Exception as e:
            print(e)
            return False, "exception"

    def extract(self, hash_key: str, time_unit: str) -> Optional[list[dict]]:
        search_url = NAVER_TREND_GRAPH_URL + hash_key
        request_header = self.get_stats_headers()
        response = requests.get(search_url, headers=request_header)
        naver_search_graph_data_value = f'<div id="graph_data" style="display:none" data-timedimension="{time_unit}">'
        if response.status_code == 200:
            try:
                re_comp_yt_info = re.compile(
                    naver_search_graph_data_value + "(\[.*?\])</div>", re.DOTALL
                )
                search_comp_yt_info = re_comp_yt_info.search(response.text)
                graph_data = search_comp_yt_info.group(1)
                yt_data = json.loads(graph_data)
                return yt_data
            except Exception:
                return None
        else:
            return None

    def crawl_trend_stat(self, query_data: dict, time_unit: str) -> list[dict]:
        headers = self.get_headers()
        response = requests.post(NAVER_TREND_HOOK_URL, data=query_data, headers=headers)
        if response.status_code == 200:
            hash_key_data = json.loads(response.text)
            hash_key = hash_key_data.get("hashKey")
            graph_date = self.extract(hash_key, time_unit)
            if graph_date is not None:
                return graph_date
            else:
                return []
        else:
            return []

    def get_trend_stat(
        self,
        start: datetime,
        end: datetime,
        time_unit: str,
        device_type: str,
        gender: str = None,
        age: str = None,
    ) -> tuple[dict, str]:
        try:
            query_data = self.get_query_hook_by_device(
                start, end, time_unit, device_type, gender, age
            )
            graph_data = self.crawl_trend_stat(query_data, time_unit)
            data = graph_data[0].get("data")
            title = graph_data[0].get("title")
            if data:
                return data, title
            else:
                return {}, ""
        except Exception:
            return {}, ""

    @staticmethod
    def convert_period(date_string: str) -> str:
        dt = datetime.strptime(date_string, "%Y%m%d")
        return dt.strftime("%Y-%m-%d")

    def search_trend_hook_stat(
        self, month_total_count: int, device_type: str = None
    ) -> tuple[bool, dict]:
        try:
            start_date, end_date = self.change_date_to_string(
                self.start_time, self.end_time, self.time_unit
            )
            data, title = self.get_trend_stat(
                self.start_time,
                self.end_time,
                self.time_unit,
                device_type,
                self.gender,
                self.age,
            )
            first_date = datetime.strptime(data[0].get("period"), "%Y%m%d")
            if self.time_unit == "date":
                if self.is_last_day:
                    count_list = self.get_count_data(data, month_total_count)
                else:
                    all_stat, _ = self.get_trend_stat(
                        self.start_time, self.yesterday, "date", device_type
                    )
                    count_data = self.get_count_data(all_stat, month_total_count)
                    count_list = [
                        {
                            "period": self.convert_period(e.get("period")),
                            "count": next(
                                (
                                    i.get("count")
                                    for i in count_data
                                    if i.get("period")
                                    == self.convert_period(e.get("period"))
                                ),
                                None,
                            ),
                        }
                        for e in data
                    ]

            elif self.time_unit == "month":
                all_stat, _ = self.get_trend_stat(
                    first_date, self.yesterday, "date", device_type
                )
                count_data = self.get_count_data(all_stat, month_total_count)
                df = pd.DataFrame(count_data)
                df = df.set_index("period")
                df.index = pd.to_datetime(df.index)
                df_group_month = df.resample("MS").agg({"count": "sum"})
                monthly_data = df_group_month.reset_index().values.tolist()
                count_list = [
                    {
                        "period": self.convert_period(e.get("period")),
                        "count": next(
                            (
                                i[1]
                                for i in monthly_data
                                if i[0].strftime("%Y-%m-%d")
                                == self.convert_period(e.get("period"))
                            ),
                            None,
                        ),
                    }
                    for e in data
                ]

            elif self.time_unit == "week":
                all_stat, _ = self.get_trend_stat(
                    first_date, self.yesterday, "date", device_type
                )
                count_data = self.get_count_data(all_stat, month_total_count)
                df = pd.DataFrame(count_data)
                df = df.set_index("period")
                df.index = pd.to_datetime(df.index)
                df_group_week = df.resample("W").agg({"count": "sum"})
                week_data = df_group_week.reset_index().values.tolist()
                count_list = [
                    {
                        "period": self.convert_period(e.get("period")),
                        "count": next(
                            (
                                i[1]
                                for i in week_data
                                if (i[0] - pd.DateOffset(6)).strftime("%Y-%m-%d")
                                == self.convert_period(e.get("period"))
                            ),
                            None,
                        ),
                    }
                    for e in data
                ]

            ratio_list = [
                {
                    "period": self.convert_period(e.get("period")),
                    "ratio": e.get("value"),
                }
                for e in data
            ]
            trend_date = {
                "startDate": start_date,
                "endDate": end_date,
                "results": [{"title": title, "data": ratio_list}],
            }
            return True, {
                "start": start_date,
                "end": end_date,
                "data": count_list,
                "trend": trend_date,
            }
        except Exception:
            return False, {}
